@@ -45,51 +45,28 @@ information will be user_dn or user_info.
 Additional classes can be found in the config module next to this one.
 """
 
+from __future__ import absolute_import, division, print_function, unicode_literals
+
 import copy
 from functools import reduce
-import ldap
 import operator
 import pprint
-import sys
-import traceback
+import re
 import warnings
 
-from django.contrib.auth.models import User, Group, Permission
+import ldap
+
+import django
 import django.conf
+from django.contrib.auth import get_user_model
+from django.contrib.auth.models import Group, Permission
 from django.core.cache import cache
 from django.core.exceptions import ImproperlyConfigured, ObjectDoesNotExist
 import django.dispatch
-try:
-    from django.utils.encoding import force_str
-except ImportError:  # Django < 1.5
-    from django.utils.encoding import smart_str as force_str
+from django.utils import six
+from django.utils.encoding import force_str
 
-# Django 1.7 Removed custom profiles
-try:
-    from django.contrib.auth.models import SiteProfileNotAvailable
-except ImportError:
-    SiteProfileNotAvailable = Exception
-
-# Support Django 1.5's custom user models
-try:
-    from django.contrib.auth import get_user_model
-
-    def get_user_username(user):
-        return user.get_username()
-except ImportError:
-    def get_user_model():
-        return User
-
-    def get_user_username(user):
-        return user.username
-
-# Small compatibility hack
-try:
-    basestring
-except NameError:
-    basestring = str
-
-from django_auth_ldap.config import ConfigurationWarning, _LDAPConfig, LDAPGroupQuery, LDAPSearch
+from django_auth_ldap.config import ConfigurationWarning, LDAPGroupQuery, LDAPSearch, _LDAPConfig
 
 
 logger = _LDAPConfig.get_logger()
@@ -97,13 +74,12 @@ logger = _LDAPConfig.get_logger()
 
 # Exported signals
 
-# Allows clients to perform custom user/profile population.
+# Allows clients to perform custom user population.
 populate_user = django.dispatch.Signal(providing_args=["user", "ldap_user"])
-populate_user_profile = django.dispatch.Signal(providing_args=["profile", "ldap_user"])
 
 # Allows clients to inspect and perform special handling of LDAPError
 # exceptions. Exceptions raised by handlers will be propagated out.
-ldap_error = django.dispatch.Signal(providing_args=['context', 'exception'])
+ldap_error = django.dispatch.Signal(providing_args=['context', 'user', 'exception'])
 
 
 class LDAPBackend(object):
@@ -130,29 +106,29 @@ class LDAPBackend(object):
         """
         Exclude certain cached properties from pickling.
         """
-        return dict((k, v) for (k, v) in self.__dict__.items()
-                    if k not in ['_settings', '_ldap'])
+        return {k: v for k, v in self.__dict__.items()
+                if k not in ['_settings', '_ldap']}
 
-    def _get_settings(self):
+    @property
+    def settings(self):
         if self._settings is None:
             self._settings = LDAPSettings(self.settings_prefix,
                                           self.default_settings)
 
         return self._settings
 
-    def _set_settings(self, settings):
+    @settings.setter
+    def settings(self, settings):
         self._settings = settings
 
-    settings = property(_get_settings, _set_settings)
-
-    def _get_ldap(self):
+    @property
+    def ldap(self):
         if self._ldap is None:
             options = getattr(django.conf.settings, 'AUTH_LDAP_GLOBAL_OPTIONS', None)
 
             self._ldap = _LDAPConfig.get_ldap(options)
 
         return self._ldap
-    ldap = property(_get_ldap)
 
     def get_user_model(self):
         """
@@ -169,9 +145,9 @@ class LDAPBackend(object):
     def authenticate(self, request=None, username=None, password=None, **kwargs):
         if bool(password) or self.settings.PERMIT_EMPTY_PASSWORD:
             ldap_user = _LDAPUser(self, username=username.strip())
-            user = ldap_user.authenticate(password)
+            user = self.authenticate_ldap_user(ldap_user, password)
         else:
-            logger.debug('Rejecting empty password for %s' % username)
+            logger.debug('Rejecting empty password for {}'.format(username))
             user = None
 
         return user
@@ -204,10 +180,12 @@ class LDAPBackend(object):
         if not hasattr(user, 'ldap_user') and self.settings.AUTHORIZE_ALL_USERS:
             _LDAPUser(self, user=user)  # This sets user.ldap_user
 
-        if hasattr(user, 'ldap_user') and (user.ldap_user.dn is not None):
-            return user.ldap_user.get_group_permissions()
+        if hasattr(user, 'ldap_user'):
+            permissions = user.ldap_user.get_group_permissions()
         else:
-            return set()
+            permissions = set()
+
+        return permissions
 
     #
     # Bonus API: populate the Django user from LDAP without authenticating.
@@ -223,14 +201,68 @@ class LDAPBackend(object):
     # Hooks for subclasses
     #
 
+    def authenticate_ldap_user(self, ldap_user, password):
+        """
+        Returns an authenticated Django user or None.
+        """
+        return ldap_user.authenticate(password)
+
+    def get_or_build_user(self, username, ldap_user):
+        """
+        This must return a (User, built) 2-tuple for the given LDAP user.
+
+        username is the Django-friendly username of the user. ldap_user.dn is
+        the user's DN and ldap_user.attrs contains all of their LDAP
+        attributes.
+
+        The returned User object may be an unsaved model instance.
+
+        """
+        if self.__class__.get_or_create_user != LDAPBackend.get_or_create_user:
+            # Older deprecated method overridden, defer to it instead.
+            warnings.warn(
+                "Method LDAPBacked.get_or_create_user() is deprecated and will "
+                "be removed in a future version. Override "
+                "LDAPBacked.get_or_build_user() instead. "
+                "LDAPBacked.get_or_build_user() does not need to save newly "
+                "built users.",
+                DeprecationWarning,
+            )
+            return self.get_or_create_user(username, ldap_user)
+
+        model = self.get_user_model()
+
+        if self.settings.USER_QUERY_FIELD:
+            query_field = self.settings.USER_QUERY_FIELD
+            query_value = ldap_user.attrs[self.settings.USER_ATTR_MAP[query_field]][0]
+            lookup = query_field
+        else:
+            query_field = model.USERNAME_FIELD
+            query_value = username.lower()
+            lookup = '{}__iexact'.format(query_field)
+
+        try:
+            user = model.objects.get(**{lookup: query_value})
+        except model.DoesNotExist:
+            user = model(**{query_field: query_value})
+            built = True
+        else:
+            built = False
+
+        return (user, built)
+
     def get_or_create_user(self, username, ldap_user):
         """
         This must return a (User, created) 2-tuple for the given LDAP user.
         username is the Django-friendly username of the user. ldap_user.dn is
-        the user's DN and ldap_user.attrs contains all of their LDAP attributes.
+        the user's DN and ldap_user.attrs contains all of their LDAP
+        attributes.
+
+        Deprecated, but remains for backwards compatibility. Remove in a future
+        release.
         """
         model = self.get_user_model()
-        username_field = getattr(model, 'USERNAME_FIELD', 'username')
+        username_field = model.USERNAME_FIELD
 
         kwargs = {
             username_field + '__iexact': username,
@@ -312,23 +344,23 @@ class _LDAPUser(object):
         Most of our properties are cached from the LDAP server. We only want to
         pickle a few crucial things.
         """
-        return dict((k, v) for (k, v) in self.__dict__.items()
-                    if k in ['backend', '_username', '_user'])
+        return {k: v for k, v in self.__dict__.items()
+                if k in ['backend', '_username', '_user']}
 
     def _set_authenticated_user(self, user):
         self._user = user
-        self._username = self.backend.django_to_ldap_username(get_user_username(user))
+        self._username = self.backend.django_to_ldap_username(user.get_username())
 
         user.ldap_user = self
         user.ldap_username = self._username
 
-    def _get_ldap(self):
+    @property
+    def ldap(self):
         return self.backend.ldap
-    ldap = property(_get_ldap)
 
-    def _get_settings(self):
+    @property
+    def settings(self):
         return self.backend.settings
-    settings = property(_get_settings)
 
     #
     # Entry points
@@ -348,16 +380,21 @@ class _LDAPUser(object):
 
             user = self._user
         except self.AuthenticationFailed as e:
-            logger.debug(u"Authentication failed for %s: %s" % (self._username, e))
+            logger.debug("Authentication failed for {}: {}".format(self._username, e))
         except ldap.LDAPError as e:
             results = ldap_error.send(self.backend.__class__,
-                                      context='authenticate', exception=e)
+                                      context='authenticate', user=self._user,
+                                      exception=e)
             if len(results) == 0:
-                logger.warning(u"Caught LDAPError while authenticating %s: %s",
-                               self._username, pprint.pformat(e))
-        except Exception:
-            logger.exception(u"Caught Exception while authenticating %s",
-                             self._username)
+                logger.warning(
+                    "Caught LDAPError while authenticating {}: {}".format(
+                        self._username, pprint.pformat(e)
+                    )
+                )
+        except Exception as e:
+            logger.warning(
+                "{} while authenticating {}".format(e, self._username)
+            )
             raise
 
         return user
@@ -372,14 +409,18 @@ class _LDAPUser(object):
 
             if self.settings.FIND_GROUP_PERMS:
                 try:
-                    self._load_group_permissions()
+                    if self.dn is not None:
+                        self._load_group_permissions()
                 except ldap.LDAPError as e:
                     results = ldap_error.send(self.backend.__class__,
                                               context='get_group_permissions',
-                                              exception=e)
+                                              user=self._user, exception=e)
                     if len(results) == 0:
-                        logger.warning("Caught LDAPError loading group permissions: %s",
-                                       pprint.pformat(e))
+                        logger.warning(
+                            "Caught LDAPError loading group permissions: {}".format(
+                                pprint.pformat(e)
+                            )
+                        )
 
         return self._group_permissions
 
@@ -398,14 +439,18 @@ class _LDAPUser(object):
             user = self._user
         except ldap.LDAPError as e:
             results = ldap_error.send(self.backend.__class__,
-                                      context='populate_user', exception=e)
+                                      context='populate_user', user=self._user,
+                                      exception=e)
             if len(results) == 0:
-                logger.warning(u"Caught LDAPError while authenticating %s: %s",
-                               self._username, pprint.pformat(e))
+                logger.warning(
+                    "Caught LDAPError while authenticating {}: {}".format(
+                        self._username, pprint.pformat(e)
+                    )
+                )
         except Exception as e:
-            logger.error(u"Caught Exception while authenticating %s: %s",
-                         self._username, pprint.pformat(e))
-            logger.error(''.join(traceback.format_tb(sys.exc_info()[2])))
+            logger.warning(
+                "{} while authenticating {}".format(e, self._username)
+            )
             raise
 
         return user
@@ -414,34 +459,34 @@ class _LDAPUser(object):
     # Public properties (callbacks). These are all lazy for performance reasons.
     #
 
-    def _get_user_dn(self):
+    @property
+    def dn(self):
         if self._user_dn is None:
             self._load_user_dn()
 
         return self._user_dn
-    dn = property(_get_user_dn)
 
-    def _get_user_attrs(self):
+    @property
+    def attrs(self):
         if self._user_attrs is None:
             self._load_user_attrs()
 
         return self._user_attrs
-    attrs = property(_get_user_attrs)
 
-    def _get_group_dns(self):
+    @property
+    def group_dns(self):
         return self._get_groups().get_group_dns()
-    group_dns = property(_get_group_dns)
 
-    def _get_group_names(self):
+    @property
+    def group_names(self):
         return self._get_groups().get_group_names()
-    group_names = property(_get_group_names)
 
-    def _get_bound_connection(self):
+    @property
+    def connection(self):
         if not self._connection_bound:
             self._bind()
 
         return self._get_connection()
-    connection = property(_get_bound_connection)
 
     #
     # Authentication
@@ -482,7 +527,7 @@ class _LDAPUser(object):
         if self._using_simple_bind_mode():
             self._user_dn = self._construct_simple_user_dn()
         else:
-            cache_key = 'django_auth_ldap.user_dn.{}'.format(self._username)
+            cache_key = valid_cache_key('django_auth_ldap.user_dn.{}'.format(self._username))
             self._user_dn = cache_get_or_set(
                 cache, cache_key, self._search_for_user_dn,
                 self.settings.GROUP_CACHE_TIMEOUT
@@ -568,41 +613,33 @@ class _LDAPUser(object):
 
         username = self.backend.ldap_to_django_username(self._username)
 
-        self._user, created = self.backend.get_or_create_user(username, self)
+        self._user, built = self.backend.get_or_build_user(username, self)
         self._user.ldap_user = self
         self._user.ldap_username = self._username
 
-        should_populate = force_populate or self.settings.ALWAYS_UPDATE_USER or created
+        should_populate = force_populate or self.settings.ALWAYS_UPDATE_USER or built
 
-        if created:
-            logger.debug("Created Django user %s", username)
+        if built:
+            logger.debug("Creating Django user {}".format(username))
             self._user.set_unusable_password()
             save_user = True
 
         if should_populate:
-            logger.debug("Populating Django user %s", username)
+            logger.debug("Populating Django user {}".format(username))
             self._populate_user()
             save_user = True
 
-        if bool(self.settings.MIRROR_GROUPS) or bool(self.settings.MIRROR_GROUPS_EXCEPT):
-            self._normalize_mirror_settings()
-            self._mirror_groups()
-
-        # Give the client a chance to finish populating the user just before
-        # saving.
-        if should_populate:
-            signal_responses = populate_user.send(self.backend.__class__, user=self._user, ldap_user=self)
-            if len(signal_responses) > 0:
-                save_user = True
+            # Give the client a chance to finish populating the user just
+            # before saving.
+            populate_user.send(self.backend.__class__, user=self._user, ldap_user=self)
 
         if save_user:
             self._user.save()
 
-        # We populate the profile after the user model is saved to give the
-        # client a chance to create the profile. Custom user models in Django
-        # 1.5 probably won't have a get_profile method.
-        if should_populate and self._should_populate_profile():
-            self._populate_and_save_user_profile()
+        # This has to wait until we're sure the user has a pk.
+        if bool(self.settings.MIRROR_GROUPS) or bool(self.settings.MIRROR_GROUPS_EXCEPT):
+            self._normalize_mirror_settings()
+            self._mirror_groups()
 
     def _populate_user(self):
         """
@@ -614,9 +651,11 @@ class _LDAPUser(object):
     def _populate_user_from_attributes(self):
         for field, attr in self.settings.USER_ATTR_MAP.items():
             try:
-                setattr(self._user, field, self.attrs[attr][0])
-            except Exception:
-                logger.warning("%s does not have a value for the attribute %s", self.dn, attr)
+                value = self.attrs[attr][0]
+            except LookupError:
+                logger.warning("{} does not have a value for the attribute {}".format(self.dn, attr))
+            else:
+                setattr(self._user, field, value)
 
     def _populate_user_from_group_memberships(self):
         for field, group_dns in self.settings.USER_FLAGS_BY_GROUP.items():
@@ -628,69 +667,6 @@ class _LDAPUser(object):
             value = query.resolve(self)
             setattr(self._user, field, value)
 
-    def _should_populate_profile(self):
-        return ((django.VERSION < (1, 7)) and
-                (getattr(django.conf.settings, 'AUTH_PROFILE_MODULE', None) is not None) and
-                hasattr(self._user, 'get_profile'))
-
-    def _populate_and_save_user_profile(self):
-        """
-        Populates a User profile object with fields from the LDAP directory.
-        """
-        try:
-            profile = self._user.get_profile()
-            save_profile = False
-
-            logger.debug("Populating Django user profile for %s", get_user_username(self._user))
-
-            save_profile = self._populate_profile_from_attributes(profile) or save_profile
-            save_profile = self._populate_profile_from_group_memberships(profile) or save_profile
-
-            signal_responses = populate_user_profile.send(self.backend.__class__, profile=profile, ldap_user=self)
-            if len(signal_responses) > 0:
-                save_profile = True
-
-            if save_profile:
-                profile.save()
-        except (SiteProfileNotAvailable, ObjectDoesNotExist):
-            logger.debug("Django user %s does not have a profile to populate", get_user_username(self._user))
-
-    def _populate_profile_from_attributes(self, profile):
-        """
-        Populate the given profile object from AUTH_LDAP_PROFILE_ATTR_MAP.
-        Returns True if the profile was modified.
-        """
-        save_profile = False
-
-        for field, attr in self.settings.PROFILE_ATTR_MAP.items():
-            try:
-                # user_attrs is a hash of lists of attribute values
-                setattr(profile, field, self.attrs[attr][0])
-                save_profile = True
-            except Exception:
-                logger.warning("%s does not have a value for the attribute %s", self.dn, attr)
-
-        return save_profile
-
-    def _populate_profile_from_group_memberships(self, profile):
-        """
-        Populate the given profile object from AUTH_LDAP_PROFILE_FLAGS_BY_GROUP.
-        Returns True if the profile was modified.
-        """
-        save_profile = False
-
-        for field, group_dns in self.settings.PROFILE_FLAGS_BY_GROUP.items():
-            try:
-                query = self._normalize_group_dns(group_dns)
-            except ValueError as e:
-                raise ImproperlyConfigured("{}: {}", self.settings._name('PROFILE_FLAGS_BY_GROUP'), e)
-
-            value = query.resolve(self)
-            setattr(profile, field, value)
-            save_profile = True
-
-        return save_profile
-
     def _normalize_group_dns(self, group_dns):
         """
         Converts one or more group DNs to an LDAPGroupQuery.
@@ -702,7 +678,7 @@ class _LDAPUser(object):
         """
         if isinstance(group_dns, LDAPGroupQuery):
             query = group_dns
-        elif isinstance(group_dns, basestring):
+        elif isinstance(group_dns, six.string_types):
             query = LDAPGroupQuery(group_dns)
         elif isinstance(group_dns, (list, tuple)) and len(group_dns) > 0:
             query = reduce(operator.or_, map(LDAPGroupQuery, group_dns))
@@ -740,7 +716,7 @@ class _LDAPUser(object):
             else:
                 raise malformed_mirror_groups_except()
 
-            if not all(isinstance(value, basestring) for value in mge):
+            if not all(isinstance(value, six.string_types) for value in mge):
                 raise malformed_mirror_groups_except()
             elif bool(mg):
                 warnings.warn(ConfigurationWarning("Ignoring {} in favor of {}".format(
@@ -757,7 +733,7 @@ class _LDAPUser(object):
             else:
                 raise malformed_mirror_groups()
 
-            if isinstance(mg, (set, frozenset)) and (not all(isinstance(value, basestring) for value in mg)):
+            if isinstance(mg, (set, frozenset)) and (not all(isinstance(value, six.string_types) for value in mg)):
                 raise malformed_mirror_groups()
 
     def _mirror_groups(self):
@@ -793,7 +769,10 @@ class _LDAPUser(object):
             new_groups = [Group.objects.get_or_create(name=name)[0] for name
                           in target_group_names if name not in existing_group_names]
 
-            self._user.groups = existing_groups + new_groups
+            if django.VERSION < (1, 11):
+                self._user.groups = existing_groups + new_groups
+            else:
+                self._user.groups.set(existing_groups + new_groups)
 
     #
     # Group information
@@ -810,7 +789,7 @@ class _LDAPUser(object):
         perms = perms.values_list('content_type__app_label', 'codename')
         perms = perms.order_by()
 
-        self._group_permissions = set(["%s.%s" % (ct, name) for ct, name in perms])
+        self._group_permissions = {"{}.{}".format(ct, name) for ct, name in perms}
 
     def _get_groups(self):
         """
@@ -909,10 +888,10 @@ class _LDAPUserGroups(object):
 
         if self._group_names is None:
             group_infos = self._get_group_infos()
-            self._group_names = set(
+            self._group_names = {
                 self._group_type.group_name_from_info(group_info)
                 for group_info in group_infos
-            )
+            }
             self._cache_attr("_group_names")
 
         return self._group_names
@@ -934,8 +913,9 @@ class _LDAPUserGroups(object):
         if is_member is None:
             is_member = (group_dn in self.get_group_dns())
 
-        logger.debug("%s is%sa member of %s", self._ldap_user.dn,
-                     is_member and " " or " not ", group_dn)
+        logger.debug("{} is{}a member of {}".format(
+            self._ldap_user.dn, is_member and " " or " not ", group_dn
+        ))
 
         return is_member
 
@@ -945,7 +925,7 @@ class _LDAPUserGroups(object):
         """
         if self._group_dns is None:
             group_infos = self._get_group_infos()
-            self._group_dns = set(group_info[0] for group_info in group_infos)
+            self._group_dns = {group_info[0] for group_info in group_infos}
 
         return self._group_dns
 
@@ -977,8 +957,10 @@ class _LDAPUserGroups(object):
         Memcache keys can't have spaces in them, so we'll remove them from the
         DN for maximum compatibility.
         """
-        dn = self._ldap_user.dn.replace(' ', '%20')
-        key = u'auth_ldap.%s.%s.%s' % (self.__class__.__name__, attr_name, dn)
+        dn = self._ldap_user.dn
+        key = valid_cache_key(
+            'auth_ldap.{}.{}.{}'.format(self.__class__.__name__, attr_name, dn)
+        )
 
         return key
 
@@ -1007,11 +989,10 @@ class LDAPSettings(object):
         'MIRROR_GROUPS': None,
         'MIRROR_GROUPS_EXCEPT': None,
         'PERMIT_EMPTY_PASSWORD': False,
-        'PROFILE_ATTR_MAP': {},
-        'PROFILE_FLAGS_BY_GROUP': {},
         'REQUIRE_GROUP': None,
         'SERVER_URI': 'ldap://localhost',
         'START_TLS': False,
+        'USER_QUERY_FIELD': None,
         'USER_ATTRLIST': None,
         'USER_ATTR_MAP': {},
         'USER_DN_TEMPLATE': None,
@@ -1036,6 +1017,15 @@ class LDAPSettings(object):
         return (self._prefix + suffix)
 
 
+def valid_cache_key(key):
+    """
+    Sanitizes a cache key for memcached.
+    """
+    key = re.sub(r'\s+', '+', key)[:250]
+
+    return key
+
+
 def cache_get_or_set(cache, key, default, timeout=None):
     """
     Backport of Django 1.9's cache.get_or_set.
