@@ -22,7 +22,6 @@
 # OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 # OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
-
 """
 This module contains classes that will be needed for configuration of LDAP
 authentication. Unlike backend.py, this is safe to import into settings.py.
@@ -30,15 +29,15 @@ Please see the docstring on the backend module for more information, including
 notes on naming conventions.
 """
 
-import ldap
+from __future__ import absolute_import, division, print_function, unicode_literals
+
 import logging
 import pprint
 
+import ldap
+
+from django.utils.encoding import force_str
 from django.utils.tree import Node
-try:
-    from django.utils.encoding import force_str
-except ImportError:  # Django < 1.5
-    from django.utils.encoding import smart_str as force_str
 
 
 class ConfigurationWarning(UserWarning):
@@ -54,6 +53,7 @@ class _LDAPConfig(object):
 
     _ldap_configured = False
 
+    @classmethod
     def get_ldap(cls, global_options=None):
         """
         Returns the ldap module. The unit test harness will assign a mock object
@@ -81,8 +81,8 @@ class _LDAPConfig(object):
             cls._ldap_configured = True
 
         return cls.ldap
-    get_ldap = classmethod(get_ldap)
 
+    @classmethod
     def get_logger(cls):
         """
         Initializes and returns our logger instance.
@@ -96,7 +96,6 @@ class _LDAPConfig(object):
             cls.logger.addHandler(NullHandler())
 
         return cls.logger
-    get_logger = classmethod(get_logger)
 
 
 # Our global logger
@@ -110,7 +109,7 @@ class LDAPSearch(object):
     documented for configuration purposes. Internal clients may use the other
     methods to refine and execute the search.
     """
-    def __init__(self, base_dn, scope, filterstr=u'(objectClass=*)', attrlist=None):
+    def __init__(self, base_dn, scope, filterstr='(objectClass=*)', attrlist=None):
         """
         These parameters are the same as the first three parameters to
         ldap.search_s.
@@ -135,9 +134,9 @@ class LDAPSearch(object):
         for name, value in term_dict.items():
             if escape:
                 value = self.ldap.filter.escape_filter_chars(value)
-            term_strings.append(u'(%s=%s)' % (name, value))
+            term_strings.append('({}={})'.format(name, value))
 
-        filterstr = u'(&%s)' % ''.join(term_strings)
+        filterstr = '(&{})'.format(''.join(term_strings))
 
         return self.__class__(self.base_dn, self.scope, filterstr)
 
@@ -147,7 +146,7 @@ class LDAPSearch(object):
         string. The caller is responsible for passing in a properly escaped
         string.
         """
-        filterstr = u'(&%s%s)' % (self.filterstr, filterstr)
+        filterstr = '(&{}{})'.format(self.filterstr, filterstr)
 
         return self.__class__(self.base_dn, self.scope, filterstr)
 
@@ -172,8 +171,11 @@ class LDAPSearch(object):
                                           self.attrlist)
         except ldap.LDAPError as e:
             results = []
-            logger.error(u"search_s('%s', %d, '%s') raised %s" %
-                         (self.base_dn, self.scope, filterstr, pprint.pformat(e)))
+            logger.error(
+                "search_s('{}', {}, '{}') raised {}".format(
+                    self.base_dn, self.scope, filterstr, pprint.pformat(e)
+                )
+            )
 
         return self._process_results(results)
 
@@ -191,13 +193,17 @@ class LDAPSearch(object):
 
         try:
             filterstr = self.filterstr % filterargs
-            msgid = connection.search(force_str(self.base_dn),
-                                      self.scope, force_str(filterstr),
-                                      self.attrlist)
+            msgid = connection.search(
+                force_str(self.base_dn), self.scope, force_str(filterstr),
+                self.attrlist
+            )
         except ldap.LDAPError as e:
             msgid = None
-            logger.error(u"search('%s', %d, '%s') raised %s" %
-                         (self.base_dn, self.scope, filterstr, pprint.pformat(e)))
+            logger.error(
+                "search('{}', {}, '{}') raised {}".format(
+                    self.base_dn, self.scope, filterstr, pprint.pformat(e)
+                )
+            )
 
         return msgid
 
@@ -211,7 +217,7 @@ class LDAPSearch(object):
                 results = []
         except ldap.LDAPError as e:
             results = []
-            logger.error(u"result(%d) raised %s" % (msgid, pprint.pformat(e)))
+            logger.error("result({}) raised {}".format(msgid, pprint.pformat(e)))
 
         return self._process_results(results)
 
@@ -228,8 +234,8 @@ class LDAPSearch(object):
             filterargs = tuple(self.ldap.filter.escape_filter_chars(value)
                                for value in filterargs)
         elif isinstance(filterargs, dict):
-            filterargs = dict((key, self.ldap.filter.escape_filter_chars(value))
-                              for key, value in filterargs.items())
+            filterargs = {key: self.ldap.filter.escape_filter_chars(value)
+                          for key, value in filterargs.items()}
         else:
             raise TypeError("filterargs must be a tuple or dict.")
 
@@ -247,9 +253,12 @@ class LDAPSearch(object):
         results = [(r[0].lower(), r[1]) for r in results]
 
         result_dns = [result[0] for result in results]
-        logger.debug(u"search_s('%s', %d, '%s') returned %d objects: %s" %
-                     (self.base_dn, self.scope, self.filterstr, len(result_dns),
-                      "; ".join(result_dns)))
+        logger.debug(
+            "search_s('{}', {}, '{}') returned {} objects: {}".format(
+                self.base_dn, self.scope, self.filterstr, len(result_dns),
+                "; ".join(result_dns)
+            )
+        )
 
         return results
 
@@ -409,12 +418,12 @@ class PosixGroupType(LDAPGroupType):
 
             if 'gidNumber' in ldap_user.attrs:
                 user_gid = ldap_user.attrs['gidNumber'][0]
-                filterstr = u'(|(gidNumber=%s)(memberUid=%s))' % (
+                filterstr = '(|(gidNumber={})(memberUid={}))'.format(
                     self.ldap.filter.escape_filter_chars(user_gid),
                     self.ldap.filter.escape_filter_chars(user_uid)
                 )
             else:
-                filterstr = u'(memberUid=%s)' % (
+                filterstr = '(memberUid={})'.format(
                     self.ldap.filter.escape_filter_chars(user_uid),
                 )
 
@@ -492,9 +501,9 @@ class NISGroupType(LDAPGroupType):
     def user_groups(self, ldap_user, group_search):
         try:
             user_uid = ldap_user.attrs['uid'][0]
-            filterstr = u'(|(nisNetgroupTriple=%s)(nisNetgroupTriple=%s))' % (
-                self.ldap.filter.escape_filter_chars('(,%s,)' % user_uid),
-                self.ldap.filter.escape_filter_chars('(-,%s,-)' % user_uid)
+            filterstr = '(|(nisNetgroupTriple={})(nisNetgroupTriple={}))'.format(
+                self.ldap.filter.escape_filter_chars('(,{},)'.format(user_uid)),
+                self.ldap.filter.escape_filter_chars('(-,{},-)'.format(user_uid))
             )
             search = group_search.search_with_additional_term_string(filterstr)
             groups = search.execute(ldap_user.connection)
@@ -508,13 +517,13 @@ class NISGroupType(LDAPGroupType):
             result = ldap_user.connection.compare_s(
                 force_str(group_dn),
                 force_str('nisNetgroupTriple'),
-                force_str('(,%s,)' % (user_uid))
+                force_str('(,{},)'.format(user_uid))
             )
             if result == 0:
                 result = ldap_user.connection.compare_s(
                     force_str(group_dn),
                     force_str('nisNetgroupTriple'),
-                    force_str('(-,%s,-)' % (user_uid))
+                    force_str('(-,{},-)'.format(user_uid))
                 )
         except (ldap.UNDEFINED_TYPE, ldap.NO_SUCH_ATTRIBUTE, KeyError, IndexError):
             result = 0
@@ -545,14 +554,14 @@ class NestedMemberDNGroupType(LDAPGroupType):
         pruned.
         """
         group_info_map = {}  # Maps group_dn to group_info of groups we've found
-        member_dn_set = set([ldap_user.dn])  # Member DNs to search with next
+        member_dn_set = {ldap_user.dn}  # Member DNs to search with next
         handled_dn_set = set()  # Member DNs that we've already searched with
 
         while len(member_dn_set) > 0:
             group_infos = self.find_groups_with_any_member(member_dn_set,
                                                            group_search,
                                                            ldap_user.connection)
-            new_group_info_map = dict([(info[0], info) for info in group_infos])
+            new_group_info_map = {info[0]: info for info in group_infos}
             group_info_map.update(new_group_info_map)
             handled_dn_set.update(member_dn_set)
 
@@ -564,11 +573,11 @@ class NestedMemberDNGroupType(LDAPGroupType):
 
     def find_groups_with_any_member(self, member_dn_set, group_search, connection):
         terms = [
-            u"(%s=%s)" % (self.member_attr, self.ldap.filter.escape_filter_chars(dn))
+            "({0}={1})".format(self.member_attr, self.ldap.filter.escape_filter_chars(dn))
             for dn in member_dn_set
         ]
 
-        filterstr = u"(|%s)" % "".join(terms)
+        filterstr = "(|{0})".format("".join(terms))
         search = group_search.search_with_additional_term_string(filterstr)
 
         return search.execute(connection)
